/*
 *
 * AttributeTree
 *
 * Handles a players individual attribute tree
 *
 *
*/

package jesse.keeblarcraft.AttributeMgr;

import java.util.ArrayList;
import java.util.List;

import jesse.keeblarcraft.AttributeMgr.Nodes.AbstractNode;
import jesse.keeblarcraft.ConfigMgr.ConfigManager;
import jesse.keeblarcraft.Utils.ChatUtil;
import jesse.keeblarcraft.Utils.ChatUtil.CONSOLE_COLOR;
import jesse.keeblarcraft.Utils.CustomExceptions.FILE_WRITE_EXCEPTION;

public class AttributeTree {

    PlayerTree playerAttributeTree = new PlayerTree();
    ConfigManager config = new ConfigManager();

    private class TreeNode {

        public TreeNode(TreeNode parent, AbstractNode newNode) {
            // If parent node is not null; that means this is not root of tree
            if (parentNode != null) {
                parentNode = parent;
            } else {
                parentNode = null;
            }

            // Set the next nodes
            thisNode = newNode;
        }

        AbstractNode thisNode;      // The attribute information

        // The below variables are used in context of the TREE (typically when drawing it). They do not affect the
        // actual code of the attribute itself
        Boolean isNodeUnlocked;     // If false then node levels are ignored and node is grayed out
        Boolean isNodeHidden;       // Default is false when drawing tree's; but if true then becomes hidden.
        Integer currentNodeLevel;   // Some nodes may have levels to unlocking them; set current level here
        Integer maxNodeLevel;       // The max node level can be set here
        Integer nodeIdentifier;     // The unique identifier of this node
        List<TreeNode> nextNodes = new ArrayList<TreeNode>(); // Leave as empty list if no more nodes
        TreeNode parentNode;      // Leave as null if this is root node
    }

    // Separate structure as this is written to the config file
    private class PlayerTree {
        String uuid;
        String username;
        Integer treeSize;
        TreeNode root;
    }

    // If parent node remains null; we will assume this is root node. If root node is already
    // taken; then this will return false. True in any other case
    public Boolean AddNewNode(AbstractNode parentNode, AbstractNode newNode) {
        Boolean success = false;

        return success;
    }

    public void ChangeNodeLevel(Integer id, Integer newLevel) {

    }

    public void DeleteNode(Integer id) {

    }

    public void DeleteNode(String nodeName) {

    }

    // Id will be retrieved from gui most likely (future us)
    public TreeNode GetNodeDetails(Integer id) {
        TreeNode ret = null;

        return ret;
    }

    // Getting a node by string will likely have poor performance
    public TreeNode GetNodeDetails(String nodeName) {
        TreeNode ret = null;

        return ret;
    }

    public AttributeTree(String uuid) {
        // DEVELOPER NOTE:
        //  If you are testing this part of the code, please be reminded that anonymous testing starts a new
        //  player instance everytime you launch. This means the UUID CAN CHANGE when you launch the
        //  game! This is not an issue with proper registered accounts in production
        Boolean existingFile = false;
        try {
            playerAttributeTree = config.GetJsonObjectFromFile("attributes/" + uuid + ".json", PlayerTree.class);
            existingFile = true;
        } catch (Exception e) {
            // Do nothing. This means the file does not exist
        }

        // In the event the above code failed out, this means a new file has to be created for the player's uuid
        if (!existingFile)
        {
            System.out.println(ChatUtil.ColoredString("Trying to create new file", CONSOLE_COLOR.BLUE));
            try {
                playerAttributeTree.uuid = uuid;
                FlashConfig();
            } catch (Exception e) {
                System.out.println(ChatUtil.ColoredString("Could not write to file", CONSOLE_COLOR.RED));
            }
        } else {
            System.out.println(ChatUtil.ColoredString("Moving on", CONSOLE_COLOR.BLUE));
        }

        // It's possible the above code will return a blank class if a file doesn't exist. This will make
        // a new file with this players uuid
        if ("".equals(playerAttributeTree.uuid)) {
            System.out.println(ChatUtil.ColoredString("Assigning new config file for this uuid. No previous existing", CONSOLE_COLOR.BLUE));
            playerAttributeTree.uuid = uuid;
        }
    }

    public void FlashConfig() {
        try {
            config.WriteToJsonFile("attributes/" + playerAttributeTree.uuid + ".json", playerAttributeTree);
        } catch (FILE_WRITE_EXCEPTION e) {
            System.out.println(ChatUtil.ColoredString("Could not flash notes configuration file", CONSOLE_COLOR.RED));
        }
    }
}
